/*
 ============================================================================
 Name		: BatteryInfo.cpp
 Author	  : Philipp Heckel
 Version	 : 1.0
 Copyright   : GPL
 Description : CBatteryInfo implementation
 ============================================================================
 */

#include "BatteryInfo.h"

CBatteryInfo::CBatteryInfo() :
	CActive(EPriorityStandard) // Standard priority
	{
	}

CBatteryInfo* CBatteryInfo::NewLC()
	{
	CBatteryInfo* self = new (ELeave) CBatteryInfo();
	CleanupStack::PushL(self);
	self->ConstructL();
	return self;
	}

CBatteryInfo* CBatteryInfo::NewL()
	{
	CBatteryInfo* self = CBatteryInfo::NewLC();
	CleanupStack::Pop(); // self;
	return self;
	}

void CBatteryInfo::ConstructL()
	{	
	CActiveScheduler::Add(this); // Add to scheduler
	iTelephony = CTelephony::NewL(); 
	}

CBatteryInfo::~CBatteryInfo()
	{
	Cancel(); // Cancel any request, if outstanding
	delete iTelephony;
	}

CTelephony::TBatteryInfoV1 CBatteryInfo::GetBatteryInfo()
	{
    if (!IsActive())
        {
		CTelephony::TBatteryInfoV1Pckg batteryInfoV1Pckg(iBatteryInfo);
        iTelephony->GetBatteryInfo(iStatus, batteryInfoV1Pckg);
        SetActive();
 
        iSchedulerWait.Start();
        // Continues here after CBatteryInfo::RunL() is called
        // by CTelephony.
        }
    else
        {
        User::Leave(KErrNotReady);
        }
    
    return iBatteryInfo;	
	}

void CBatteryInfo::RunL()
	{
	// This is called when GetBatteryInfo() completes
	iSchedulerWait.AsyncStop();
	}

void CBatteryInfo::DoCancel()
    {
    // This cancels the  GetBatteryInfo() method
    iSchedulerWait.AsyncStop();
    }
