/*
 ============================================================================
 Name		: TCPDaemon.cpp
 Author	  : Philipp Heckel
 Version	 : 1.0
 Copyright   : GPL
 Description : CTCPDaemon implementation
 ============================================================================
 */

#include <TCPDaemon.h>
#include <Application.h>

#include <in_sock.h>
#include <e32cons.h>
#include <es_sock.h>

CTCPDaemon::CTCPDaemon()
	{
	// No implementation required
	}

CTCPDaemon::~CTCPDaemon()
	{
	}

CTCPDaemon* CTCPDaemon::NewLC(CApplication* aApplication)
	{
	CTCPDaemon* self = new (ELeave) CTCPDaemon();
	CleanupStack::PushL(self);
	self->ConstructL(aApplication);
	return self;
	}

CTCPDaemon* CTCPDaemon::NewL(CApplication* aApplication)
	{
	CTCPDaemon* self = CTCPDaemon::NewLC(aApplication);
	CleanupStack::Pop(); // self;
	return self;
	}

void CTCPDaemon::ConstructL(CApplication* aApplication)
	{
		iApplication = aApplication;
		iConsole = iApplication->GetConsole();		
	}

void CTCPDaemon::Listen() 
	{
	TUint KTestPort=8100;
	TInetAddr addr(KInetAddrLoop, KTestPort);
	RSocketServ socketServ;
	RSocket listener;
	RSocket blank;

	iConsole->Printf(_L("Listening on 127.0.0.1:%d!\n\n"), KTestPort);
	
	User::LeaveIfError(socketServ.Connect());
	CleanupClosePushL(socketServ);//if function leaves ensure socket serv session closes
	User::LeaveIfError(listener.Open(socketServ, KAfInet,KSockStream, KProtocolInetTcp));
	User::LeaveIfError(listener.Bind(addr));  
	User::LeaveIfError(listener.Listen(1));

	//iConsole->Printf(_L("Listen successful!\n"));

	TRequestStatus status;
	TSockXfrLength dummyLength;
	TBool running = ETrue;

	_LIT8(KBadCommand, "Bad command");
	_LIT8(KClosing, "Server closing");

	TBuf8<256> buffer; // 256 works!

	while(running) 
		{
		blank.Open(socketServ);
		//iConsole->Printf(_L("Accepting connections (blocking) ...\n"));
		listener.Accept(blank, status); 
		User::WaitForRequest(status);

		if(status != KErrNone)
			User::Leave(KErrGeneral);

		//iConsole->Printf(_L("Accepted connection!!\n"));		
		blank.RecvOneOrMore(buffer, 0, status, dummyLength);
		User::WaitForRequest(status);	

		if(status != KErrNone)
			User::Leave(KErrGeneral);

		// ACTUAL PROTOCOL COMMANDS ////////////////////////////////////////////
		
		if (buffer[0] == 'w') 
			{
			iConsole->Printf(_L("[w] Scanning wireless networks ...\n"));
			iApplication->GetWirelessNetworks(blank); // Needs to be signed + add. capabilities
			//iApplication->GetWirelessNetworksTest(blank); // No add. capabilities
			} 		

		else if( (buffer[0] == 'b') ) 
			{
			iConsole->Printf(_L("[b] Retrieving battery status ...\n"));
			iApplication->GetBatteryStatus(blank);
			} 		

		else if( (buffer[0] == 'c') ) 
			{
			iConsole->Printf(_L("[c] Closing server!!\n"));
			running = EFalse;	

			buffer.Copy(KClosing);
			
			blank.Write(buffer, status);
			User::WaitForRequest(status); 

			if(status != KErrNone)
				User::Leave(KErrGeneral);			
			} 
		
		// HUMAN READABLE COMMANDS /////////////////////////////////////////////
		
		else if (buffer[0] == 'W') 
			{
			iConsole->Printf(_L("[W] Scanning wireless networks ...\n"));
			iApplication->GetWirelessNetworksHumanReadable(&buffer);
			
			blank.Write(buffer, status);
			User::WaitForRequest(status); 

			if(status != KErrNone)
				User::Leave(KErrGeneral);			
			} 		
		
		else if( (buffer[0] == 'B') ) 
			{
			iConsole->Printf(_L("[B] Retrieving battery status ...\n"));
			iApplication->GetBatteryStatusHumanReadable(&buffer); // TODO
			
			blank.Write(buffer, status);
			User::WaitForRequest(status); 

			if(status != KErrNone)
				User::Leave(KErrGeneral);			
			} 		

		else 
			{
			iConsole->Printf(_L("Bad command!!\n"));
			buffer.Copy(KBadCommand);
			
			blank.Write(buffer, status);
			User::WaitForRequest(status); 

			if(status != KErrNone)
				User::Leave(KErrGeneral);			
			}

		blank.Close();	
		
		}//end of while loop

	CleanupStack::Pop(&socketServ);
	socketServ.Close();		

	}

void CTCPDaemon::ProcessList(TDes8* aBuffer) 
	{
	_LIT8(KSpace, " ");

	TBuf<256> res;
	TFindProcess findP;

	aBuffer->Zero();
	//Get only the first five processes, otherwise it goes on for ages!

	for(TInt i = 0; i < 5; i++) 

		{ 

		findP.Next(res);

		//check that we are not overflowing aBuffer

		if( (aBuffer->Size() + res.Size()) < 256 )

			{

			aBuffer->Append(res); 

			aBuffer->Append(KSpace);

			}

		else

			{

			break;

			}

		}

	//while(FP.Next(res)==KErrNone) { aBuffer->Append(res); aBuffer->Append(KSpace); }

	}
