/*
 ============================================================================
 Name		: WirelessSensor.cpp
 Author	  : Philipp Heckel
 Version	 : 1.0
 Copyright   : GPL
 Description : CWirelessSensor implementation
 Source      : Partially from http://wiki.forum.nokia.com/index.php/How_to_retrieve_the_SSID_and_BSSID_using_WLAN_INFO_API
 
 CAPABILITY NetworkServices ReadUserData WriteUserData ReadDeviceData
 
 
 ============================================================================
 */

#include "WirelessSensor.h"
//#include <wlanmgmtclient.h> // For ??, link against: ?
//#include <wlanscaninfo.h> // For CWlanScanInfo, link against: ?
#include <apengineconsts.h> // for KErrNullPointerPassed



CWirelessSensor::CWirelessSensor() :
	CActive(EPriorityStandard) // Standard priority
	{
	}

CWirelessSensor* CWirelessSensor::NewLC()
	{
	CWirelessSensor* self = new (ELeave) CWirelessSensor();
	CleanupStack::PushL(self);
	self->ConstructL();
	return self;
	}

CWirelessSensor* CWirelessSensor::NewL()
	{
	CWirelessSensor* self = CWirelessSensor::NewLC();
	CleanupStack::Pop(); // self;
	return self;
	}

void CWirelessSensor::ConstructL()
	{
	CActiveScheduler::Add(this); // Add to scheduler
	iWLANMgmtClient = CWlanMgmtClient::NewL();
	}

CWirelessSensor::~CWirelessSensor()
	{
	Cancel(); // Cancel any request, if outstanding
	}

/**
 * This function is used to list WLAN Info with WLAN Info API
 * 
 * @infoBuff - when function returns it holds the WLAN Information
 */
CArrayFixFlat<CWirelessNetwork>* CWirelessSensor::GetWirelessNetworks()
	{
		CArrayFixFlat<CWirelessNetwork>* networks = new (ELeave) CArrayFixFlat<CWirelessNetwork>(5);		
		CWlanScanInfo* iScanInfo = CWlanScanInfo::NewL();
		
		iWLANMgmtClient->GetScanResults(*iScanInfo); 
	 
		for(iScanInfo->First(); !iScanInfo->IsDone(); iScanInfo->Next() )
			{
			CWirelessNetwork* wifi = CWirelessNetwork::NewL();
		 
			// Retrieve BSSID
			TWlanBssid bssid;
			iScanInfo->Bssid(bssid);		
			wifi->SetBSSID(bssid);
				
			// Get transmision level
			TInt8 rxLevel = iScanInfo->RXLevel();
			wifi->SetSignalStrength(rxLevel);
		 
			// Get SSID
			TWlanSsid ssid;
			TInt err = GetSSID(iScanInfo, ssid);
			if(err == KErrNone)
				{
					wifi->SetSSID(ssid);
				}
		
			// Add to list
			networks->AppendL(*wifi);
			}
		
		return networks;
	}

/**
 * Function that retrieves SSID of current scaned network
 * 
 * The SSID is the Information Element with ID = 0
 * The SSID can be at most 32 bytes, so the aSSID should have the capacity for 32 bytes
 * 
 * Returns KErrNone if successful or the type of error ocurred
 */
TInt CWirelessSensor::GetSSID(CWlanScanInfo *scanInfo, TDes8 &aSSID)
	{
	TInt error;
	TUint8 ie;
	TUint8 length;
	TUint8 vdata[40];
	const TUint8 *data = &vdata[0];
 
	if(scanInfo == NULL)
		return KErrNullPointerPassed;
 
	if(aSSID.MaxLength() < (TInt) KWlanMaxSsidLength)
		return KErrNoMemory;
 
 
	if((error = scanInfo->FirstIE(ie, length, &data)) == KErrNone)
	{
		if(ie == 0)
		{
			aSSID.Copy(data, length);
			return KErrNone;
		}
	}
	else
		return error;
 
	while((error = scanInfo->NextIE(ie, length, &data)) == KErrNone)
	{
		if(ie == 0)
		{
			aSSID.Copy(data, length);
			return KErrNone;
		}
	}
 
	return error;
	}

void CWirelessSensor::RunL()
	{
	// This is called when GetBatteryInfo() completes
	iSchedulerWait.AsyncStop();
	}

void CWirelessSensor::DoCancel()
    {
    // This cancels the  GetBatteryInfo() method
    iSchedulerWait.AsyncStop();
    }

